
/* Define to prevent recursive inclusion -------------------------------------*/
#include "sd_mmc_spi.h"
#include <MC9S12XS128.h> 

/*******************************************************************************
* Function Name  : SD_MMC_SPI_Init
* Description    : SD_MMC_SPI_Init
* Input          : None
* Output         : None
* Return         : zero init success, non-zero init error
*******************************************************************************/
unsigned char SD_MMC_SPI_Init(void)
{
  unsigned int i;
  unsigned char response;
  
    /* initialize SPI with lowest frequency */
    SD_MMC_Low_Speed();
    
    /* card needs 74 cycles minimum to start up */
    for( i = 0; i < 10; ++i)
    {
        /* wait 8 clock cycles */
        SD_MMC_ReadWrite_Byte(0x00);
    }

    /* address card */
    SD_MMC_SPI_SELECT();

    /* reset card */

    for( i = 0; ; ++i)
    {
        response = SD_MMC_Send_Command(CMD_GO_IDLE_STATE , 0 );
        if( response == 0x01 )
            break;

        if(i == 0x1ff)
        {
            SD_MMC_SPI_DESELECT();
            return 1;
        }
    }
    
    /* wait for card to get ready */
    for(i = 0; ; ++i)
    {
        response = SD_MMC_Send_Command(CMD_SEND_OP_COND, 0);
        if(!(response & (1 << R1_IDLE_STATE)))
            break;

        if(i == 0x7fff)
        {
            SD_MMC_SPI_DESELECT();
            return 1;
        }
    }

    /* set block size to 512 bytes */
    if(SD_MMC_Send_Command(CMD_SET_BLOCKLEN, 512))
    {
        SD_MMC_SPI_DESELECT();
        return 1;
    }

    /* deaddress card */
    SD_MMC_SPI_DESELECT();

    /* switch to highest SPI frequency possible */
    //SD_MMC_High_Speed();
    
    return 0;
}

/*******************************************************************************
* Function Name  : SD_MMC_ReadWrite_Byte
* Description    : SD_MMC_ReadWrite_Byte
* Input          : the char to send
* Output         : None
* Return         : the char received
*******************************************************************************/
unsigned char SD_MMC_ReadWrite_Byte(unsigned char ch)
{
  unsigned char spi_tmp;
  
  while (!SPI0SR_SPTEF);
  SPI0DRL = ch;
  while (!SPI0SR_SPIF);
  spi_tmp = SPI0DRL;
  return (spi_tmp);
}

/*******************************************************************************
* Function Name  : SD_MMC_Low_Speed
* Description    : SD_MMC_Low_Speed
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void SD_MMC_Low_Speed(void)
{
     // set low speed
    SPI0CR1 = 0x5e; //CPOL=1,CPHA=1 0b01011110
    SPI0CR2 = 0x10; //0b00010000
    SPI0BR  = 0x07;  // 80M / 256 = 312.5k
}

/*******************************************************************************
* Function Name  : SD_MMC_High_Speed
* Description    : SD_MMC_High_Speed
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void SD_MMC_High_Speed(void)
{
    SPI0BR  = 0x00;  // 80M / 2 = 40M 
}

/*******************************************************************************
* Function Name  : SD_MMC_Send_Command
* Description    : SD_MMC_Send_Command
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
unsigned char SD_MMC_Send_Command(unsigned char cmd, unsigned long arg)
{
        unsigned char Response;
        unsigned char Retry = 0;

        SD_MMC_ReadWrite_Byte(0xff);

        SD_MMC_SPI_SELECT();

        //ֱд
        SD_MMC_ReadWrite_Byte(cmd | 0x40); 
        SD_MMC_ReadWrite_Byte(arg >> 24);
        SD_MMC_ReadWrite_Byte(arg >> 16);
        SD_MMC_ReadWrite_Byte(arg >> 8);
        SD_MMC_ReadWrite_Byte(arg);
        SD_MMC_ReadWrite_Byte(cmd == CMD_GO_IDLE_STATE ? 0x95 : 0xff);


        do{
                // ȴӦ,ӦĿʼλΪ0 
                Response = SD_MMC_ReadWrite_Byte(0xff);
                Retry++;
        }while( ((Response&0x80)!=0) && (Retry < 200) );

        SD_MMC_SPI_DESELECT();

        return Response;  //״ֵ̬
}


/*******************************************************************************
* Function Name  : SD_MMC_Read_Single_Block
* Description    : SD_MMC_Read_Single_Block
* Input          : sector number and buffer data point
* Output         : None
* Return         : zero success, non-zero error
*******************************************************************************/

unsigned char SD_MMC_Read_Single_Block(unsigned long sector, unsigned char * buffer)
{
  unsigned char Response;
  unsigned int i;
  unsigned long Retry = 0;

   // send read command
  Response = SD_MMC_Send_Command(CMD_READ_SINGLE_BLOCK, sector<<9);

  if(Response != 0x00)
          return Response;

  SD_MMC_SPI_SELECT();

  // start byte 0xfe
  while(SD_MMC_ReadWrite_Byte(0xff) != 0xfe)
  {
    if(++Retry > 0xffffffe)
    {
        SD_MMC_SPI_DESELECT();
        return 1; //timeout
    }
  }

  for(i = 0; i < 512; ++i)
  {
          //512
          *buffer++ = SD_MMC_ReadWrite_Byte(0xff);
  }

  SD_MMC_ReadWrite_Byte(0xff);  //αcrc
  SD_MMC_ReadWrite_Byte(0xff);  //αcrc

  SD_MMC_SPI_DESELECT();
  SD_MMC_ReadWrite_Byte(0xff);  // extra 8 CLK

  return 0;
}


/*******************************************************************************
* Function Name  : SD_MMC_Write_Single_Block
* Description    : SD_MMC_Write_Single_Block
* Input          : sector number and buffer data point
* Output         : None
* Return         : zero success, non-zero error.
*******************************************************************************/
unsigned char SD_MMC_Write_Single_Block(unsigned long sector, unsigned char * buffer)
{
unsigned char Response;
unsigned int i;
unsigned int retry=0;

//&ETH;&acute;&Atilde;&Aacute;&icirc; send write command
Response = SD_MMC_Send_Command(CMD_WRITE_SINGLE_BLOCK, sector<<9);
if(Response != 0x00)
return 3; //

SD_MMC_SPI_SELECT();

SD_MMC_ReadWrite_Byte(0xff);
SD_MMC_ReadWrite_Byte(0xff);
SD_MMC_ReadWrite_Byte(0xff);

//&cent;&iquest;&ordf;&Ecirc;&frac14;&ucirc; start byte 0xfe
SD_MMC_ReadWrite_Byte(0xFE);

//&Euml;&Iacute;512&Ouml;&frac12;&Uacute;&Ecirc;&yacute;&frac34;&Yacute; send 512 bytes data
for(i=0; i<512; i++) //512
{
SD_MMC_ReadWrite_Byte(*buffer++);
} 

SD_MMC_ReadWrite_Byte(0xff); //dummy crc
SD_MMC_ReadWrite_Byte(0xff); //dummy crc

Response = SD_MMC_ReadWrite_Byte(0xff);

//&micro;&Egrave;&acute;&yacute;&Ecirc;&Ccedil;&ntilde;&sup3;&Eacute;&sup1;&brvbar; judge if it successful
if( (Response&0x1f) != 0x05)
{
SD_MMC_SPI_DESELECT();
return 2; //Response
}
//&micro;&Egrave;&acute;&yacute;&sup2;&Ugrave;&Iacute; wait no busy
while(SD_MMC_ReadWrite_Byte(0xff) != 0x00)
{
if(retry++ > 0xfffe)
{
SD_MMC_SPI_DESELECT();
return 1;
}
} 
SD_MMC_SPI_DESELECT();
SD_MMC_ReadWrite_Byte(0xff);// extra 8 CLK 
// SD_MMC_SPI_SELECT(); һΪд ΪWJԭ
for( i = 0; ; ++i)
{
Response = SD_MMC_Send_Command(CMD_GO_IDLE_STATE , 0 );
if( Response == 0x01 )
break;

if(i == 0x1ff)
{
SD_MMC_SPI_DESELECT();
return 1;
}
}
for(i = 0; ; ++i)
{
Response = SD_MMC_Send_Command(CMD_SEND_OP_COND, 0);
if(!(Response & (1 << R1_IDLE_STATE)))
break;

if(i == 0x7fff)
{
SD_MMC_SPI_DESELECT();
return 1;
}
}
return 0;
}
  
   /*
unsigned char SD_MMC_Write_Single_Block(unsigned long sector, unsigned char * buffer)
{
	unsigned char Response;
	unsigned int i;
	unsigned int retry=0;
	
        //д  send write command
	Response = SD_MMC_Send_Command(CMD_WRITE_SINGLE_BLOCK, sector<<9);
	if(Response != 0x00)
	  return Response;

	SD_MMC_SPI_SELECT();
	
	SD_MMC_ReadWrite_Byte(0xff);
	SD_MMC_ReadWrite_Byte(0xff);
	SD_MMC_ReadWrite_Byte(0xff);

        //ʼ	start byte 0xfe
	SD_MMC_ReadWrite_Byte(0xfe);
	
        //512ֽ	send 512 bytes data
	for(i=0; i<512; i++)
	{
		SD_MMC_ReadWrite_Byte(*buffer++);
	}
	
	SD_MMC_ReadWrite_Byte(0xff); //dummy crc
	SD_MMC_ReadWrite_Byte(0xff); //dummy crc
	
	Response = SD_MMC_ReadWrite_Byte(0xff);
	
        //ȴǷɹ	judge if it successful
	if( (Response&0x1f) != 0x05)
	{
		SD_MMC_SPI_DESELECT();
		return Response;
	}
	//ȴ  wait no busy
	while(SD_MMC_ReadWrite_Byte(0xff) != 0x00)
	{
		if(retry++ > 0xfffe)
		{
			SD_MMC_SPI_DESELECT();
			return 1;
		}
	}
	SD_MMC_SPI_DESELECT();
	SD_MMC_ReadWrite_Byte(0xff);// extra 8 CLK

	return 0;
}
     */

